@@ -17,7 +17,7 @@ __all__ = ['dataframe']
 class dataframe(object):
     '''
     A simple data frame, based on simple lists, for simply storing simple data.
-    
+
     Example usage:
         a = dataframe(cols=['x','y'],data=[[1238,2],[384,5],[666,7]]) # Create data frame
         print(a)['x'] # Print out a column
@@ -37,9 +37,9 @@ class dataframe(object):
         a.getrow(1) # Return the row starting with value '1'
         a.rmrow(); print(a) # Remove last row
         a.rmrow(1238); print(a) # Remove the row starting with element '3'
-    
+
     Works for both numeric and non-numeric data.
-    
+
     Version: 2019sep24
     '''
 
@@ -48,17 +48,17 @@ class dataframe(object):
         self.data = None
         self.make(cols=cols, data=data, nrows=nrows)
         return None
-    
-    
+
+
     def __repr__(self, spacing=2):
         ''' spacing = space between columns '''
         if not self.cols: # No keys, give up
             return '<empty dataframe>'
-        
+
         else: # Go for it
             outputlist = odict()
             outputformats = odict()
-            
+
             # Gather data
             nrows = self.nrows
             for c,col in enumerate(self.cols):
@@ -70,33 +70,33 @@ class dataframe(object):
                         maxlen = max(maxlen, len(output))
                         outputlist[col].append(output)
                 outputformats[col] = '%'+'%i'%(maxlen+spacing)+'s'
-            
+
             ndigits = (np.floor(np.log10(max(1,nrows)))+1) # Don't allow 0 rows
             indformat = '%%%is' % ndigits # Choose the right number of digits to print
-            
+
             # Assemble output
             output = indformat % '' # Empty column for index
             for col in self.cols: # Print out header
                 output += outputformats[col] % col
             output += '\n'
-            
+
             for ind in range(nrows): # Loop over rows to print out
                 output += indformat % ut.flexstr(ind)
                 for col in self.cols: # Print out data
                     output += outputformats[col] % outputlist[col][ind]
                 if ind<nrows-1: output += '\n'
-            
+
             return output
-    
-    
+
+
     def _val2row(self, value=None):
         ''' Convert a list, array, or dictionary to the right format for appending to a dataframe '''
         if isinstance(value, dict):
             output = np.zeros(self.ncols, dtype=object)
             for c,col in enumerate(self.cols):
-                try: 
+                try:
                     output[c] = value[col]
-                except: 
+                except:
                     errormsg = 'Entry for column %s not found; keys you supplied are: %s' % (col, value.keys())
                     raise Exception(errormsg)
             output = np.array(output, dtype=object)
@@ -109,8 +109,8 @@ class dataframe(object):
                 errormsg = 'Row has wrong length (%s supplied, %s expected)' % (len(value), self.ncols)
                 raise Exception(errormsg)
         return output
-    
-    
+
+
     def _sanitizecol(self, col, die=True):
         ''' Take None or a string and return the index of the column '''
         if col is None:
@@ -120,7 +120,7 @@ class dataframe(object):
                 output = self.cols.index(col) # Convert to index
             except Exception as E:
                 errormsg = 'Could not get index of column "%s"; columns are:\n%s\n%s' % (col, '\n'.join(self.cols), str(E))
-                if die: 
+                if die:
                     raise Exception(errormsg)
                 else:
                     print(errormsg)
@@ -135,8 +135,8 @@ class dataframe(object):
                 print(errormsg)
                 output = None
         return output
-    
-    
+
+
     @staticmethod
     def _cast(arr):
         ''' Attempt to cast an array to different data types, to avoid having completely numeric arrays of object type '''
@@ -147,8 +147,8 @@ class dataframe(object):
             except: # If anything goes wrong, do nothing
                 pass
         return output
-        
-    
+
+
     def __getitem__(self, key=None, die=True, cast=True):
         ''' Simple method for returning; see self.get() for a version based on col and row '''
         if ut.isstring(key): # e.g. df['a']
@@ -196,8 +196,8 @@ class dataframe(object):
                 print(errormsg)
                 output = None
         return output
-        
-    
+
+
     def __setitem__(self, key, value=None):
         if value is None:
             value = np.zeros(self.nrows, dtype=object)
@@ -224,7 +224,7 @@ class dataframe(object):
                 self.data = np.hstack((self.data, np.array(val_arr, dtype=object)))
         elif ut.isnumber(key):
             value = self._val2row(value) # Make sure it's in the correct format
-            if len(value) != self.ncols: 
+            if len(value) != self.ncols:
                 errormsg = 'Vector has incorrect length (%i vs. %i)' % (len(value), self.ncols)
                 raise Exception(errormsg)
             rowindex = int(key)
@@ -246,8 +246,8 @@ class dataframe(object):
                 errormsg = 'Could not insert element (%s,%s) in dataframe of shape %s: %s' % (key[0], key[1], self.data.shape, str(E))
                 raise Exception(errormsg)
         return None
-    
-    
+
+
     def __copy__(self, *args, **kwargs):
         ''' Warning, there is a bug in Numpy such that deepcopy(array) != array if array contains lists '''
         newcols = ut.dcp(self.cols)
@@ -255,26 +255,26 @@ class dataframe(object):
         copied = ut.dcp(listdata)
         newdata = np.array(copied, dtype=object)
         return dataframe(cols=newcols, data=newdata)
-    
-    
+
+
     def __deepcopy__(self, *args, **kwargs):
         ''' As above '''
         return self.__copy__(*args, **kwargs)
-    
-    
+
+
     def __eq__(self, other):
         ''' If columns and data match, the dataframes match '''
         if not isinstance(other, dataframe): return False
         if  self.cols != other.cols:         return False
         if (self.data != other.data).any():  return False
         return True
-    
-    
+
+
     def __len__(self):
         ''' Return the number of rows in the dataframe '''
         return self.nrows
-    
-    
+
+
     def make(self, cols=None, data=None, nrows=None):
         '''
         Creates a dataframe from the supplied input data. Usage examples:
@@ -285,7 +285,7 @@ class dataframe(object):
             df = sc.dataframe(['a','b','c'], [[1,2,3],[4,5,6]])
             df = sc.dataframe(cols=['a','b','c'], data=[[1,2,3],[4,5,6]])
         '''
-        
+
         # Handle columns
         if nrows is None:
             nrows = 0
@@ -295,18 +295,23 @@ class dataframe(object):
         elif cols is None and data is not None: # Shouldn't happen, but if it does, swap inputs
             cols = data
             data = None
-        
+
         if not ut.isstring(pd): # This is done because if pandas import fails, it replaces it with a string
             if isinstance(cols, pd.DataFrame): # It's actually a Pandas dataframe
                 self.pandas(df=cols)
                 return None # We're done
         else:
             print(pd)
-        
-        if not ut.checktype(cols, 'listlike'):
+
+        # A dictionary is supplied: assume keys are columns, and the rest is the data
+        if isinstance(cols, dict):
+            data = [col for col in cols.values()]
+            cols = list(cols.keys())
+
+        elif not ut.checktype(cols, 'listlike'):
             errormsg = 'Inputs to dataframe must be list, tuple, or array, not %s' % (type(cols))
             raise Exception(errormsg)
-        
+
         # Handle data
         if data is None:
             if np.ndim(cols)==2 and np.shape(cols)[0]>1: # It's a 2D array with more than one row: treat first as header
@@ -334,13 +339,13 @@ class dataframe(object):
         else:
             errormsg = 'Number of columns (%s) does not match array shape (%s)' % (len(cols), data.shape)
             raise Exception(errormsg)
-        
-        # Store it        
+
+        # Store it
         self.cols = list(cols)
         self.data = data
         return None
-    
-    
+
+
     def get(self, cols=None, rows=None, asarray=True, cast=True):
         '''
         More complicated way of getting data from a dataframe; example:
@@ -357,7 +362,7 @@ class dataframe(object):
             rowindices = Ellipsis
         else:
             rowindices = rows
-        
+
         output = self.data[:,colindices][rowindices,:] # Split up so can handle non-consecutive entries in either
         if output.size == 1: output = output[0] # If it's a single element, return the value rather than the array
         if asarray:
@@ -375,15 +380,15 @@ class dataframe(object):
         self.data = np.vstack((self.data[:rowindex,:], self.data[rowindex+1:,:]))
         if returnval: return thisrow
         else:         return None
-    
-    
+
+
     def append(self, value):
         ''' Add a row to the end of the data frame '''
         value = self._val2row(value) # Make sure it's in the correct format
         self.data = np.vstack((self.data, np.array(value, dtype=object)))
         return None
-    
-    
+
+
     @property
     def ncols(self):
         ''' Get the number of columns in the data frame '''
@@ -400,17 +405,17 @@ class dataframe(object):
         ''' Get the number of rows in the data frame '''
         try:    return self.data.shape[0]
         except: return 0 # If it didn't work, probably because it's empty
-    
-    
+
+
     @property
     def shape(self):
         ''' Equivalent to the shape of the data array, minus the headers '''
         return (self.nrows, self.ncols)
-    
+
     def addcol(self, key=None, value=None):
         ''' Add a new column to the data frame -- for consistency only '''
         self.__setitem__(key, value)
-    
+
     def rmcol(self, key, die=True):
         ''' Remove a column or columns from the data frame '''
         cols = ut.promotetolist(key)
@@ -424,7 +429,7 @@ class dataframe(object):
                 self.cols.pop(colindex) # Remove from list of columns
                 self.data = np.hstack((self.data[:,:colindex], self.data[:,colindex+1:])) # Remove from data
         return None
-    
+
     def addrow(self, value=None, overwrite=True, col=None, reverse=False):
         ''' Like append, but removes duplicates in the first column and resorts '''
         value = self._val2row(value) # Make sure it's in the correct format
@@ -434,24 +439,24 @@ class dataframe(object):
         else: self.data[index,:] = value # If it exists already, just replace it
         self.sort(col=col, reverse=reverse) # Sort
         return None
-    
+
     def _rowindex(self, key=None, col=None, die=False):
         ''' Get the sanitized row index for a given key and column '''
         col = self._sanitizecol(col)
         coldata = self.data[:,col] # Get data for this column
         if key is None: key = coldata[-1] # If not supplied, pick the last element
         try:    index = coldata.tolist().index(key) # Try to find duplicates
-        except: 
+        except:
             if die: raise Exception('Item %s not found; choices are: %s' % (key, coldata))
             else:   return None
         return index
-        
+
     def rmrow(self, key=None, col=None, returnval=False, die=True):
         ''' Like pop, but removes by matching the first column instead of the index -- WARNING, messy '''
         index = self._rowindex(key=key, col=col, die=die)
         if index is not None: self.pop(index, returnval=returnval)
         return None
-    
+
     def _diffindices(self, indices=None):
         ''' For a given set of indices, get the inverse, in set-speak '''
         if indices is None: indices = []
@@ -459,7 +464,7 @@ class dataframe(object):
         all_set = set(np.arange(self.nrows))
         diff_set = np.array(list(all_set - ind_set))
         return diff_set
-    
+
     def rmrows(self, indices=None, copy=None):
         ''' Remove rows by index -- WARNING, messy '''
         if copy is None: copy = False
@@ -471,7 +476,7 @@ class dataframe(object):
         else:
             self.data = keep_data
             return None
-    
+
     def replace(self, col=None, old=None, new=None):
         ''' Replace all of one value in a column with a new value '''
         col = self._sanitizecol(col)
@@ -479,20 +484,20 @@ class dataframe(object):
         inds = ma.findinds(coldata==old)
         self.data[inds,col] = new
         return None
-        
-    
+
+
     def _todict(self, row):
         ''' Return row as a dict rather than as an array '''
-        if len(row)!=len(self.cols): 
+        if len(row)!=len(self.cols):
             errormsg = 'Length mismatch between "%s" and "%s"' % (row, self.cols)
             raise Exception(errormsg)
         rowdict = odict(zip(self.cols, row))
         return rowdict
-    
+
     def findrow(self, key=None, col=None, default=None, closest=False, die=False, asdict=False):
         '''
         Return a row by searching for a matching value.
-        
+
         Arguments:
             key = the value to look for
             col = the column to look for this value in
@@ -500,7 +505,7 @@ class dataframe(object):
             closest = whether or not to return the closest row (overrides default and die)
             die = whether to raise an exception if the value is not found
             asdict = whether to return results as dict rather than list
-        
+
         Example:
             df = dataframe(cols=['year','val'],data=[[2016,0.3],[2017,0.5]])
             df.findrow(2016) # returns array([2016, 0.3], dtype=object)
@@ -508,7 +513,7 @@ class dataframe(object):
             df.findrow(2013, closest=True) # returns array([2016, 0.3], dtype=object)
             df.findrow(2016, asdict=True) # returns {'year':2016, 'val':0.3}
         '''
-        if not closest: # Usual case, get 
+        if not closest: # Usual case, get
             index = self._rowindex(key=key, col=col, die=(die and default is None))
         else:
             col = self._sanitizecol(col)
@@ -521,8 +526,8 @@ class dataframe(object):
         else:
             thisrow = default # If not found, return as default
         return thisrow
-    
-    
+
+
     def findrows(self, key=None, col=None, asarray=False):
         ''' A method like get() or indexing, but returns a dataframe by default -- WARNING, redundant? '''
         indices = self.rowindex(key=key, col=col)
@@ -533,14 +538,14 @@ class dataframe(object):
             df = dataframe(cols=self.cols, data=arr)
             return df
 
-    
+
     def rowindex(self, key=None, col=None):
         ''' Return the indices of all rows matching the given key in a given column. '''
         col = self._sanitizecol(col)
         coldata = self.data[:,col] # Get data for this column
         indices = ma.findinds(coldata==key)
         return indices
-        
+
     def _filterrows(self, key=None, col=None, keep=True, verbose=False, copy=None):
         ''' Filter rows and either keep the ones matching, or discard them '''
         indices = self.rowindex(key=key, col=col)
@@ -548,15 +553,15 @@ class dataframe(object):
         if verbose: print('Dataframe filtering: %s rows removed based on key="%s", column="%s"' % (len(indices), key, col))
         output = self.rmrows(indices=indices, copy=copy)
         return output
-    
+
     def filter_in(self, key=None, col=None, verbose=False, copy=None):
         '''Keep only rows matching a criterion (in place) '''
         return self._filterrows(key=key, col=col, keep=True, verbose=verbose, copy=copy)
-    
+
     def filter_out(self, key=None, col=None, verbose=False, copy=None):
         '''Remove rows matching a criterion (in place) '''
         return self._filterrows(key=key, col=col, keep=False, verbose=verbose, copy=copy)
-    
+
     def filtercols(self, cols=None, die=True, copy=None):
         if copy is None: copy = False
         if cols is None: cols = ut.dcp(self.cols) # By default, do nothing
@@ -581,14 +586,14 @@ class dataframe(object):
             self.cols = cols # These should be in the correct order
             self.data = ordered_data
             return None
-        
+
     def insert(self, row=0, value=None):
         ''' Insert a row at the specified location '''
         rowindex = int(row)
         value = self._val2row(value) # Make sure it's in the correct format
         self.data = np.vstack((self.data[:rowindex,:], value, self.data[rowindex:,:]))
         return None
-    
+
     def sort(self, col=None, reverse=False):
         ''' Sort the data frame by the specified column(s)'''
         if col is None:
@@ -601,7 +606,7 @@ class dataframe(object):
             if reverse: sortorder = np.array(list(reversed(sortorder)))
             self.data = self.data[sortorder,:]
         return sortorder
-    
+
     def sortcols(self, sortorder=None, reverse=False):
         if sortorder is None:
             sortorder = np.argsort(self.cols, kind='mergesort')
@@ -609,19 +614,19 @@ class dataframe(object):
         self.cols = list(np.array(self.cols)[sortorder])
         self.data = self.data[:,sortorder]
         return sortorder
-        
+
     def jsonify(self, cols=None, rows=None, header=None, die=True):
         ''' Export the dataframe to a JSON-compatible format '''
-        
+
         # Handle input arguments
         if cols   is None: cols   = self.cols # Use all columns by default
         if rows   is None: rows   = list(range(self.nrows)) # Use all rows by default
         if header is None: header = True # Include headers
-        
+
         # Optionally filter columns after making a copy
         exportdf = ut.dcp(self)
         exportdf.filtercols(cols)
-        
+
         # Handle output
         output = []
         if header: output.append(exportdf.cols)
@@ -635,7 +640,7 @@ class dataframe(object):
                     pass # This has already been handled by the validation above
             output.append(thisrow)
         return output
-    
+
     def pandas(self, df=None):
         ''' Function to export to pandas (if no argument) or import from pandas (with an argument) '''
         if ut.isstring(pd):
@@ -650,7 +655,7 @@ class dataframe(object):
             self.cols = list(df.columns)
             self.data = np.array(df, dtype=object)
             return None
-    
+
     def export(self, filename=None, cols=None, close=True):
         ''' Export to Excel '''
         from . import sc_fileio as io # Optional import
