@@ -24,23 +24,14 @@ from glob import glob
 from gzip import GzipFile
 from zipfile import ZipFile
 from contextlib import closing
-from collections import OrderedDict
 from pathlib import Path
+from io import BytesIO as IO
+import pickle as pkl
+import copyreg as cpreg
 from . import sc_utils as ut
 from .sc_odict import odict
 from .sc_dataframe import dataframe
 
-# Handle types and Python 2/3 compatibility
-import six
-if six.PY3: # Python 3
-    from io import BytesIO as IO
-    import pickle as pkl
-    import copyreg as cpreg
-else: # Python 2
-    from cStringIO import StringIO as IO
-    import cPickle as pkl
-    import copy_reg as cpreg
-
 
 
 
@@ -90,10 +81,21 @@ def loadstr(string=None, verbose=False, die=None):
     return obj
 
 
-def saveobj(filename=None, obj=None, compresslevel=5, verbose=0, folder=None, method='pickle'):
+def saveobj(filename=None, obj=None, compresslevel=5, verbose=0, folder=None, method='pickle', *args, **kwargs):
     '''
     Save an object to file -- use compression 5 by default, since more is much slower but not much smaller.
-    Once saved, can be loaded with loadobj() (q.v.).
+    Once saved, can be loaded with sc.loadobj().
+
+    Args:
+        filename (str or Path): the filename to save to; if str, passed to sc.makefilepath()
+        obj (literally anything): the object to save
+        compresslevel (int): the level of gzip compression
+        verbose (int): detail to print
+        folder (str): passed to sc.makefilepath()
+        method (str): whether to use pickle (default) or dill
+        args (list): passed to pickle.dumps()
+        kwargs (dict): passed to pickle.dumps()
+
 
     Usage:
         myobj = ['this', 'is', 'a', 'weird', {'object':44}]
@@ -114,12 +116,13 @@ def saveobj(filename=None, obj=None, compresslevel=5, verbose=0, folder=None, me
         else: # Otherwise, try pickle
             try:
                 if verbose>=2: print('Saving as pickle...')
-                savepickle(fileobj, obj) # Use pickle
+                savepickle(fileobj, obj, *args, **kwargs) # Use pickle
             except Exception as E:
                 if verbose>=2: print('Exception when saving as pickle (%s), saving as dill...' % repr(E))
-                savedill(fileobj, obj) # ...but use Dill if that fails
+                savedill(fileobj, obj, *args, **kwargs) # ...but use Dill if that fails
 
-    if verbose and filename: print('Object saved to "%s"' % filename)
+    if verbose and filename:
+        print('Object saved to "%s"' % filename)
 
     if filename:
         return filename
@@ -129,6 +132,7 @@ def saveobj(filename=None, obj=None, compresslevel=5, verbose=0, folder=None, me
 
 
 def dumpstr(obj=None):
+    ''' Dump an object as a bytes-like string '''
     with closing(IO()) as output: # Open a "fake file."
         with GzipFile(fileobj=output, mode='wb') as fileobj:  # Open a Gzip-compressing way to write to this "file."
             try:    savepickle(fileobj, obj) # Use pickle
@@ -144,7 +148,7 @@ def dumpstr(obj=None):
 ### Other file functions
 ##############################################################################
 
-__all__ += ['loadtext', 'savetext', 'savezip', 'getfilelist', 'sanitizefilename', 'makefilepath']
+__all__ += ['loadtext', 'savetext', 'savezip', 'getfilelist', 'sanitizefilename', 'makefilepath', 'thisdir']
 
 
 
@@ -200,36 +204,51 @@ def sanitizefilename(rawfilename):
     return filtername # Return the sanitized file name.
 
 
-def makefilepath(filename=None, folder=None, ext=None, default=None, split=False, abspath=True, makedirs=True, verbose=False, sanitize=False):
+def makefilepath(filename=None, folder=None, ext=None, default=None, split=False, abspath=True, makedirs=True, checkexists=None, sanitize=False, die=True, verbose=False):
     '''
-    Utility for taking a filename and folder -- or not -- and generating a valid path from them.
+    Utility for taking a filename and folder -- or not -- and generating a
+    valid path from them. By default, this function will combine a filename and
+    folder using os.path.join, create the folder(s) if needed with os.makedirs,
+    and return the absolute path.
 
-    Inputs:
-        filename = the filename, or full file path, to save to -- in which case this utility does nothing
-        folder = the name of the folder to be prepended to the filename
-        ext = the extension to ensure the file has
-        default = a name or list of names to use if filename is None
-        split = whether to return the path and filename separately
-        makedirs = whether or not to make the folders to save into if they don't exist
-        verbose = how much detail to print
+    Args:
+        filename (str or Path): the filename, or full file path, to save to -- in which case this utility does nothing
+        folder (str, Path, or list): the name of the folder to be prepended to the filename; if a list, fed to os.path.join()
+        ext (str): the extension to ensure the file has
+        default (str or list): a name or list of names to use if filename is None
+        split (bool): whether to return the path and filename separately
+        makedirs (bool): whether or not to make the folders to save into if they don't exist
+        checkexists (bool): if False/True, raises an exception if the path does/doesn't exist
+        sanitize (bool): whether or not to remove special characters from the path; see sc.sanitizefilename() for details
+        verbose (bool): how much detail to print
 
-    Example:
-        makefilepath(filename=None, folder='./congee', ext='prj', default=[project.filename, project.name], split=True, abspath=True, makedirs=True)
+    Returns:
+        filepath (str): the validated path (or the folder and filename if split=True)
+
+    Simple example:
+        filepath = sc.makefilepath('myfile.obj') # Equivalent to os.path.abspath(os.path.expanduser('myfile.obj'))
+
+    Complex example:
+        filepath = makefilepath(filename=None, folder='./congee', ext='prj', default=[project.filename, project.name], split=True, abspath=True, makedirs=True)
 
     Assuming project.filename is None and project.name is "soggyrice" and ./congee doesn't exist:
         * Makes folder ./congee
         * Returns e.g. ('/home/myname/congee', 'soggyrice.prj')
 
-    Actual code example from project.py:
-        fullpath = makefilepath(filename=filename, folder=folder, default=[self.filename, self.name], ext='prj')
-
-    Version: 2018sep22
+    Version: 2020apr11
     '''
 
     # Initialize
     filefolder = '' # The folder the file will be located in
     filebasename = '' # The filename
 
+    if isinstance(filename, Path):
+        filename = str(filename)
+    if isinstance(folder, Path): # If it's a path object, convert to string
+        folder = str(folder)
+    if isinstance(folder, list): # It's a list, join together
+        folder = os.path.join(*folder)
+
     # Process filename
     if filename is None:
         defaultnames = ut.promotetolist(default) # Loop over list of default names
@@ -254,36 +273,90 @@ def makefilepath(filename=None, folder=None, ext=None, default=None, split=False
         filefolder = folder
     if abspath: # Convert to absolute path
         filefolder = os.path.abspath(os.path.expanduser(filefolder))
-    if makedirs: # Make sure folder exists
-        try: os.makedirs(filefolder)
-        except: pass
+
+    # Make sure folder exists
+    if makedirs:
+        try:
+            os.makedirs(filefolder, exist_ok=True)
+        except Exception as E:
+            if die:
+                raise E
+            else:
+                print(f'Could not create folders: {str(E)}')
+
+    # Create the full path
+    filepath = os.path.join(filefolder, filebasename)
+
+    # Optionally check if it exists
+    if checkexists is not None:
+        exists = os.path.exists(filepath)
+        errormsg = ''
+        if exists and not checkexists:
+            errormsg = f'File {filepath} should not exist, but it does'
+            if die:
+                raise FileExistsError(errormsg)
+        if not exists and checkexists:
+            errormsg = f'File {filepath} should exist, but it does not'
+            if die:
+                raise FileNotFoundError(errormsg)
+        if errormsg:
+            print(errormsg)
+
+    # Decide on output
     if verbose:
-        print('From filename="%s", folder="%s", abspath="%s", makedirs="%s", made folder name "%s"' % (filename, folder, abspath, makedirs, filefolder))
+        print(f'From filename="{filename}", folder="{folder}", made path name "{filepath}"')
+    if split:
+        output = filefolder, filebasename
+    else:
+        output = filepath
 
-    fullfile = os.path.join(filefolder, filebasename) # And the full thing
+    return output
 
-    if split: return filefolder, filebasename
-    else:     return fullfile # Or can do os.path.split() on output
 
+def thisdir(file, *args, **kwargs):
+    '''
+    Tiny helper function to get the folder for a file, usually the current file.
+
+    Args:
+        file (str): the file to get the directory from; usually __file__
+        args (list): passed to os.path.join()
+        kwargs (dict): also passed to os.path.join()
+
+    Returns:
+        filepath (str): the full path to the folder (or filename if additional arguments are given)
+
+    Examples:
+        thisdir = sc.thisdir(__file__)
+        file_in_same_dir = sc.thisdir(__file__, 'new_file.txt')
+    '''
+    folder = os.path.abspath(os.path.dirname(file))
+    filepath = os.path.join(folder, *args, **kwargs)
+    return filepath
 
 
 ##############################################################################
 ### JSON functions
 ##############################################################################
 
-__all__ += ['sanitizejson', 'jsonify', 'loadjson', 'savejson']
+__all__ += ['sanitizejson', 'jsonify', 'loadjson', 'savejson', 'jsonpickle', 'jsonunpickle']
 
 
 def sanitizejson(obj, verbose=True, die=False, tostring=False, **kwargs):
     """
     This is the main conversion function for Python data-structures into
     JSON-compatible data structures.
-    Use this as much as possible to guard against data corruption!
+
     Args:
-        obj: almost any kind of data structure that is a combination
-            of list, numpy.ndarray, odicts, etc.
+        obj (any): almost any kind of data structure that is a combination of list, numpy.ndarray, odicts, etc.
+        verbose (bool): level of detail to print
+        die (bool): whether or not to raise an exception if conversion failed (otherwise, return a string)
+        tostring (bool): whether to return a string representation of the sanitized object instead of the object itself
+        kwargs (dict): passed to json.dumps() if tostring=True
+
     Returns:
-        A converted dict/list/value that should be JSON compatible
+        object (any or str): the converted object that should be JSON compatible, or its representation as a string if tostring=True
+
+    Version: 2020apr11
     """
     if obj is None: # Return None unchanged
         output = None
@@ -295,8 +368,15 @@ def sanitizejson(obj, verbose=True, die=False, tostring=False, **kwargs):
         if np.isnan(obj): # It's nan, so return None
             output = None
         else:
-            if isinstance(obj, (int, np.int64)): output = int(obj) # It's an integer
-            else:                                output = float(obj)# It's something else, treat it as a float
+            if isinstance(obj, (int, np.int64)):
+                output = int(obj) # It's an integer
+            else:
+                output = float(obj)# It's something else, treat it as a float
+
+    elif ut.isstring(obj): # It's a string of some kind
+        try:    string = str(obj) # Try to convert it to ascii
+        except: string = obj # Give up and use original
+        output = string
 
     elif isinstance(obj, np.ndarray): # It's an array, iterate recursively
         if obj.shape: output = [sanitizejson(p) for p in list(obj)] # Handle most cases, incluing e.g. array([5])
@@ -305,33 +385,38 @@ def sanitizejson(obj, verbose=True, die=False, tostring=False, **kwargs):
     elif isinstance(obj, (list, set, tuple)): # It's another kind of interable, so iterate recurisevly
         output = [sanitizejson(p) for p in list(obj)]
 
-    elif isinstance(obj, dict): # Treat all dictionaries as ordered dictionaries
-        output = OrderedDict()
-        for key,val in obj.items():
-            output[str(key)] = sanitizejson(val)
+    elif isinstance(obj, dict): # It's a dictionary, so iterate over the items
+        output = {str(key):sanitizejson(val) for key,val in obj.items()}
 
-    elif isinstance(obj, datetime.datetime):
+    elif isinstance(obj, (datetime.time, datetime.date, datetime.datetime)):
         output = str(obj)
 
     elif isinstance(obj, uuid.UUID):
         output = str(obj)
 
-    elif ut.isstring(obj): # It's a string of some kind
-        try:    string = str(obj) # Try to convert it to ascii
-        except: string = obj # Give up and use original
-        output = string
+    elif callable(getattr(obj, 'to_dict', None)): # Handle e.g. pandas, where we want to return the object, not the string
+        output = obj.to_dict()
+
+    elif callable(getattr(obj, 'to_json', None)):
+        output = obj.to_json()
+
+    elif callable(getattr(obj, 'toJSON', None)):
+        output = obj.toJSON()
 
     else: # None of the above
         try:
-            output = json.loads(json.dumps(obj)) # Try passing it through jsonification
+            output = jsonpickle(obj)
         except Exception as E:
             errormsg = 'Could not sanitize "%s" %s (%s), converting to string instead' % (obj, type(obj), str(E))
             if die:       raise Exception(errormsg)
             elif verbose: print(errormsg)
             output = str(obj)
 
-    if tostring: return json.dumps(output, **kwargs)
-    else:        return output
+    # Convert to string if desired
+    if tostring:
+        output = json.dumps(output, **kwargs)
+
+    return output
 
 
 
@@ -340,11 +425,34 @@ def jsonify(*args, **kwargs):
     return sanitizejson(*args, **kwargs)
 
 
-def loadjson(filename=None, folder=None, **kwargs):
-    ''' Convenience function for reading a JSON file '''
-    filename = makefilepath(filename=filename, folder=folder)
-    with open(filename) as f:
-        output = json.load(f, **kwargs)
+def loadjson(filename=None, folder=None, string=None, fromfile=True, **kwargs):
+    '''
+    Convenience function for reading a JSON file (or string).
+
+    Args:
+        filename (str): the file to load, or the JSON object if using positional arguments
+        folder (str): folder if not part of the filename
+        string (str): if not loading from a file, a string representation of the JSON
+        fromfile (bool): whether or not to load from file
+        kwargs (dict): passed to json.load()
+
+    Returns:
+        output (dict): the JSON object
+
+    '''
+    if fromfile:
+        filename = makefilepath(filename=filename, folder=folder)
+        try:
+            with open(filename) as f:
+                output = json.load(f, **kwargs)
+        except FileNotFoundError as E:
+            print('Warning: file not found. Use fromfile=False if not loading from a file')
+            raise E
+    else:
+        if string is None and filename is not None:
+            string = filename # Swap arguments
+        output = json.loads(string, **kwargs)
+
     return output
 
 
@@ -357,6 +465,39 @@ def savejson(filename=None, obj=None, folder=None, **kwargs):
     return None
 
 
+def jsonpickle(obj, tostring=False):
+    ''' Use jsonpickle to return a representation of an object '''
+    import jsonpickle as jp
+    import jsonpickle.ext.numpy as jsonpickle_numpy
+    import jsonpickle.ext.pandas as jsonpickle_pandas
+    jsonpickle_numpy.register_handlers()
+    jsonpickle_pandas.register_handlers()
+
+    if tostring:
+        output = jp.dumps(obj)
+    else:
+        pickler = jp.pickler.Pickler()
+        output = pickler.flatten(obj)
+
+    return output
+
+
+def jsonunpickle(json):
+    ''' Use jsonunpickle to restore an object '''
+    import jsonpickle as jp
+    import jsonpickle.ext.numpy as jsonpickle_numpy
+    import jsonpickle.ext.pandas as jsonpickle_pandas
+    jsonpickle_numpy.register_handlers()
+    jsonpickle_pandas.register_handlers()
+
+    if isinstance(json, str):
+        output = jp.loads(json)
+    else:
+        unpickler = jp.unpickler.Unpickler()
+        output = unpickler.restore(json)
+
+    return output
+
 
 ##############################################################################
 ### Spreadsheet functions
@@ -415,6 +556,9 @@ class Blobject(object):
                 output = f.read()
             return output
 
+        if isinstance(source, Path):  # If it's a path object, convert to string
+            source = str(source)
+
         if source is None:
             if self.bytes is not None:
                 self.blob = read_bin(self.bytes)
@@ -438,21 +582,11 @@ class Blobject(object):
 
     def save(self, filename=None):
         ''' This function writes the spreadsheet to a file on disk. '''
-        if filename is None:
-            if self.filename is not None:
-                filename = self.filename
-            elif self.name is not None:
-                if self.name.endswith('.xlsx'):
-                    filename = self.name
-                else:
-                    filename = self.name + '.xlsx'
-            else:
-                filename = 'spreadsheet.xlsx' # Come up with a terrible default name
         filepath = makefilepath(filename=filename)
         with open(filepath, mode='wb') as f:
             f.write(self.blob)
         self.filename = filename
-        print('Spreadsheet saved to %s.' % filepath)
+        print('Object saved to %s.' % filepath)
         return filepath
 
     def tofile(self, output=True):
@@ -630,9 +764,9 @@ class Spreadsheet(Blobject):
 
         return None
 
-
-    pass
-
+    def save(self, filename='spreadsheet.xlsx'):
+        filepath = makefilepath(filename=filename, ext='xlsx')
+        super().save(filepath)
 
 def loadspreadsheet(filename=None, folder=None, fileobj=None, sheetname=None, sheetnum=None, asdataframe=None, header=True, cells=None):
     '''
@@ -924,19 +1058,21 @@ def unpickler(string=None, filename=None, filestring=None, die=None, verbose=Fal
     return obj
 
 
-def savepickle(fileobj=None, obj=None):
+def savepickle(fileobj=None, obj=None, protocol=None, *args, **kwargs):
         ''' Use pickle to do the salty work '''
-        fileobj.write(pkl.dumps(obj, protocol=-1))
+        if protocol is None:
+            protocol = 4
+        fileobj.write(pkl.dumps(obj, protocol=protocol, *args, **kwargs))
         return None
 
 
-def savedill(fileobj=None, obj=None):
+def savedill(fileobj=None, obj=None, *args, **kwargs):
     ''' Use dill to do the sour work '''
     try:
         import dill # Optional Sciris dependency
     except ModuleNotFoundError as e:
         raise Exception('The "dill" Python package is not available; please install manually') from e
-    fileobj.write(dill.dumps(obj, protocol=-1))
+    fileobj.write(dill.dumps(obj, protocol=-1, *args, **kwargs))
     return None
 
 
@@ -1106,21 +1242,14 @@ def loadobj2to3(filename=None, filestring=None, recursionlimit=None):
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-if six.PY2:
-    import cPickle
-    class _UniversalPicklingError(pickle.PicklingError, cPickle.PicklingError):
-        pass
-else:
-    _UniversalPicklingError = pickle.PicklingError
+_UniversalPicklingError = pickle.PicklingError
 
 def pickleMethod(method):
-    if six.PY3: return (unpickleMethod, (method.__name__,         method.__self__, method.__self__.__class__))
-    else:       return (unpickleMethod, (method.im_func.__name__, method.im_self,  method.im_class))
+    return (unpickleMethod, (method.__name__,         method.__self__, method.__self__.__class__))
 
 def _methodFunction(classObject, methodName):
     methodObject = getattr(classObject, methodName)
-    if six.PY3: return methodObject
-    else:       return methodObject.im_func
+    return methodObject
 
 def unpickleMethod(im_name, im_self, im_class):
     if im_self is None:
@@ -1133,8 +1262,7 @@ def unpickleMethod(im_name, im_self, im_class):
             raise
         return unpickleMethod(im_name, im_self, im_self.__class__)
     else:
-        if six.PY3: maybeClass = ()
-        else:       maybeClass = tuple([im_class])
+        maybeClass = ()
         bound = types.MethodType(methodFunction, im_self, *maybeClass)
         return bound
 
